/* ============================= C MeatAxe ==================================
   File:        $Id: bsissub.c,v 1.1.1.1 2000/01/07 16:42:26 mringe Exp $
   Comment:     Bit string incidence relation.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.bs
!function BsIsSub "Bit string incidence relation"
!arg a
    First bit string.
!arg b
    Second bit string.
!return
    $1$ if $a\subseteq b$, 0 if not, $-1$ on error.
!description
    This function checks if one bit string is contained in a second bit 
    string. By definition, $a\subseteq b$ if and only if every bit which is 
    set in $a$ is also set in $b$. Both bit strings must have the same size.
!seealso 
-------------------------------------------------------------------------- }}}*/

int BsIsSub(const BitString_t *a, const BitString_t *b)

{	
    register int i;
    register const unsigned long *ap, *bp;

    /* Check the arguments
       ------------------- */
    if (!BsIsValid(a))
    {
	MTX_ERROR1("a: %E",MTX_ERR_BADARG);
	return -1;
    }
    if (!BsIsValid(b))
    {
	MTX_ERROR1("b: %E",MTX_ERR_BADARG);
	return -1;
    }
    if (a->Size != b->Size)
    {
	MTX_ERROR1("%E",MTX_ERR_INCOMPAT);
	return -1;
    }

    /* Calculate the result
       -------------------- */
    ap = a->Data;
    bp = b->Data;
    for (i = a->BufSize; i > 0; --i, ++ap, ++bp)
    {
	if ((*ap ^ (*ap & *bp)) != 0)
	    return 0;
    }
    return 1;
}

