/* ============================= C MeatAxe ==================================
   File:        $Id: bsminus.c,v 1.1.1.1 2000/01/07 16:42:26 mringe Exp $
   Comment:     Bit string subtraction.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.bs
!function BsMinus "Union of two bit strings"
!arg dest
    Destination bit string.
!arg src
    Source bit string, or-ed to |dest|.
!return
    $0$ on success, $-1$ on error.
!description
    \verb"BsMinus()" computes the (set theoretical) difference of two bit 
    strings, i.e., each bit in the result is set if and only if it is set in 
    \verb"dest" and not set in \verb"src". \verb"src" and \verb"dest" must 
    have have equal size. 
    The result is stored in \verb"dest" and overwrites the previous value.
!seealso 
-------------------------------------------------------------------------- }}}*/

int BsMinus(BitString_t *dest, const BitString_t *src)

{
    register int i;
    register unsigned long *dp;
    register const unsigned long *sp;

    /* Check the arguments
       ------------------- */
    if (!BsIsValid(dest))
    {
	MTX_ERROR1("dest: %E",MTX_ERR_BADARG);
	return -1;
    }
    if (!BsIsValid(src))
    {
	MTX_ERROR1("src: %E",MTX_ERR_BADARG);
	return -1;
    }
    if (dest->Size != src->Size)
    {
	MTX_ERROR1("%E",MTX_ERR_INCOMPAT);
	return -1;
    }

    /* AND operation
       ------------- */
    dp = dest->Data;
    sp = src->Data;
    for (i = src->BufSize; i > 0; --i)
	*dp++ &= ~*sp++;

    return 0;
}

