/* ============================= C MeatAxe ==================================
   File:        $Id: imatwrite.c,v 1.1.1.1 2000/01/07 16:42:26 mringe Exp $
   Comment:     Write an integer matrix into a file.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <stdlib.h>

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.imat
!function ImatWrite "Write an integer matrix to a file"
!arg mat
    Pointer to the matrix.
!arg f
    Pointer to the file.
!return
    The function returns $0$ on success and $-1$ on error.
!description
    This function writes an integer matrix to a file.
!seealso ImatSave
-------------------------------------------------------------------------- }}}*/

int ImatWrite(const IntMatrix_t *mat, FILE *f)

{
    long hdr[3];

    if (!ImatIsValid(mat)) 
	return -1;
    hdr[0] = -8;    /* HACK: T_IMAT in 2.3 */
    hdr[1] = mat->Nor;
    hdr[2] = mat->Noc;
    if (SysWriteLong(f,hdr,3) != 3) 
    {
	MTX_ERROR("Cannot write header");
	return -1;
    }
    if (SysWriteLong(f,mat->Data,mat->Nor*mat->Noc) != mat->Nor*mat->Noc)
	return -1;
    return 0;
}




/*{{{ --------------------------------------------------------------------------
!function ImatSave "Write an integer matrix to a file"
!arg mat
    Pointer to the matrix.
!arg file_name
    File name.
!return
    $0$ on success, $-1$ on error.
!description
    This function writes an integer matrix to a named file. If the file
    exists, it is destroyed.
!seealso ImatWrite
-------------------------------------------------------------------------- }}}*/

int ImatSave(const IntMatrix_t *mat, const char *file_name)

{
    FILE *f;
    int rc;

    if (!ImatIsValid(mat)) 
	return -1;
    f = SysFopen(file_name,FM_CREATE);
    if (f == NULL)
	return -1;
    rc = ImatWrite(mat,f);
    fclose(f);
    return rc;
}

