/* ============================= C MeatAxe ==================================
   File:        $Id: matadd.c,v 1.1.1.1 2000/01/07 16:42:26 mringe Exp $
   Comment:     Add two matrices
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"

/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

#ifdef DEBUG
MTX_DEFINE_FILE_INFO 
#endif



/*{{{ --------------------------------------------------------------------------
!section obj.mat
!function MatAdd "Add a matrix"
!arg dest
    Matrix to add to.
!arg src
    Matrix to add.
!return
    The function returns |dest|, or |NULL| on error.
!description
    This function adds |src| to |dest|.  The matrices must be compatible,
    i.e., they must be over the same field and they must have the same size.
!seealso MatAddMul MatMul MatMulScalar
-------------------------------------------------------------------------- }}}*/

Matrix_t *MatAdd(Matrix_t *dest, const Matrix_t *src)
{
    PTR dp, sp;
    register int n;

    /* Check arguments
       --------------- */
#ifdef DEBUG
    if (!MatIsValid(src) || !MatIsValid(dest))
	return NULL;
    if (dest->Field != src->Field || dest->Nor != src->Nor || 
	dest->Noc != src->Noc)
	return MTX_ERROR1("%E",MTX_ERR_INCOMPAT), NULL;
#endif

    /* Add <src> to <dest>
       ------------------- */
    dp = dest->Data;
    sp = src->Data;
    FfSetField(src->Field);
    FfSetNoc(src->Noc);
    for (n = src->Nor; n > 0; --n)
    {
	FfAddRow(dp,sp);
	FfStepPtr(&dp);
	FfStepPtr(&sp);
    }

    /* Delete the pivot table
       ---------------------- */
    Mat_DeletePivotTable(dest);

    return dest;
}
