/* ============================= C MeatAxe ==================================
   File:        $Id: mmulscal.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Multiply matrix by scalar.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"



/*{{{ --------------------------------------------------------------------------
!section obj.mat
!function MatMulScalar "Calculate the multiple of a matrix"
!synopsis 
    Matrix_t *MatMulScalar(Matrix_t *dest, FEL coeff);
!arg dest
    Pointer to the matrix.
!arg coeff
    Value to multiply with.
!return
    The function returns |dest|.
!description
    This function multiplies a matrix by a constant value.
!seealso MatAdd MatAddMul MatMul 
-------------------------------------------------------------------------- }}}*/

Matrix_t *MatMulScalar(Matrix_t *dest, FEL coeff)

{
#ifdef DEBUG
    if (!MatIsValid(dest))
	return NULL;
#endif

    if (coeff == FF_ONE)
    {
	/* Nothing to do */
    }
    else
    {
	PTR dp = dest->Data;
	int n;
	FfSetField(dest->Field);
	FfSetNoc(dest->Noc);
	for (n = dest->Nor; n > 0; --n)
	{
	    FfMulRow(dp,coeff);
	    FfStepPtr(&dp);
	}
    }
    return dest;
}

