/* ============================= C MeatAxe ==================================
   File:        $Id: permdup.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Duplicate a permutation.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO


/*{{{ --------------------------------------------------------------------------
!section obj.perm
!function PermDup "Duplicate a permutation"
!arg src
    Pointer to the permutation.
!return
    Pointer to a copy of |src| or |NULL| on error.
!description
    This function creates a copy of an existing permutation. 
!seealso
-------------------------------------------------------------------------- }}}*/

Perm_t *PermDup(const Perm_t *src)

{
    Perm_t *p;

    if (!PermIsValid(src))
    {
	MTX_ERROR1("src: %E",MTX_ERR_BADARG);
	return NULL;
    }
    p = PermAlloc(src->Degree);
    if (p == NULL) 
    {
	MTX_ERROR("Cannot allocate result");
	return NULL;
    }
    memcpy(p->Data,src->Data,(size_t) src->Degree * sizeof(long));
    return p;
}


