/* ============================= C MeatAxe ==================================
   File:        $Id: permmul.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Multiplication of permutations.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.perm
!function PermMul "Multiply permutations"
!arg dest
    Pointer to the first permutation.
!arg src
    Pointer to the second permutation.
!return
    The function returns |dest|, or |NULL| on error.
!description
    This function multiplies |dest| from the right by |src|. Both 
    permutations must have the same degree.
!seealso
-------------------------------------------------------------------------- }}}*/

Perm_t *PermMul(Perm_t *dest, const Perm_t *src)

{
    register long i;
    register long *d, *s;

    /* Check arguments 
       --------------- */
    if (!PermIsValid(dest) || !PermIsValid(src))
	return NULL;
    if (dest->Degree != src->Degree)
    {
	MTX_ERROR1("%E",MTX_ERR_INCOMPAT);
    	return NULL;
    }

    /* Multiply
       -------- */
    d = dest->Data;
    s = src->Data;
    for (i = dest->Degree; i > 0; --i)
    {
	*d = s[*d];
	++d;
    }
    return dest;
}




