/* ============================= C MeatAxe ==================================
   File:        $Id: polderive.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Polynomial derivation.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.poly
!function PolDerive "Derive a polynomials"
!arg pol
    Pointer to the polynomial.
!return
    Returns |pol|.
!description
    This function derives a polynomial. Note that the derived polynomial is
    stored in |pol|, replacing the original polynomial. The following piece of
    code shows how to keep the original polynomial intact while calculating
    the derivative:
    ||    Poly_t *pol, *der;
    ||    ...
    ||    der = PolDerive(PolDup(pol));
!seealso
-------------------------------------------------------------------------- }}}*/

Poly_t *PolDerive(Poly_t *pol)

{
    int i, maxdeg = -1;
    register FEL *buf;
    FEL f = FF_ZERO;

    /* Check the argument
       ------------------ */
    if (!PolIsValid(pol))
    {
	MTX_ERROR1("%E",MTX_ERR_BADARG);
	return NULL;
    }

    buf = pol->Data;
    FfSetField(pol->Field);
    for (i = 0; i < pol->Degree; ++i)
    {
	f = FfAdd(f,FF_ONE);
	buf[i] = FfMul(buf[i+1],f);
	if (buf[i] != FF_ZERO) 
	    maxdeg = i;
    }
    pol->Degree = maxdeg;
    return pol;
}
      

