/* ============================= C MeatAxe ==================================
   File:        $Id: polmul.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Multiply two polynomials.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.poly
!function PolMul "Multiply polynomials"
!arg dest
    Pointer to the first polynomial.
!arg src
    Pointer to the second polynomial.
!return
    The function returns |dest|, or |NULL| on error.
!description
    Thus function multiplies |src| by |dest| and returns |dest|.
    The polynomials must be over the same field. Otherwise, the return
    value is |NULL|.
!seealso
-------------------------------------------------------------------------- }}}*/

Poly_t *PolMul(Poly_t *dest, const Poly_t *src)

{
    FEL *x, *y, *d, *s;
    int di, si;
    size_t xdeg;

    /* Check arguments
       --------------- */
    if (!PolIsValid(src) || !PolIsValid(dest))
	return NULL;
    if (dest->Field != src->Field) 
    {
	MTX_ERROR1("%E",MTX_ERR_INCOMPAT);
	return NULL;
    }

    /* Handle special cases: dest = 0, src = 0
       --------------------------------------- */
    if (dest->Degree == -1)
	return dest;
    if (src->Degree == -1)
    {
	dest->Degree = -1;
	return dest;
    }

    
    d = dest->Data;
    s = src->Data;
    xdeg = src->Degree + dest->Degree;
    FfSetField(src->Field);

    /* Allocate memory for the result
       ------------------------------ */
    x = NALLOC(FEL,xdeg+1);
    if (x == NULL) 
    {
	MTX_ERROR("Cannot allocate result");
	return NULL;
    }
    for (di = xdeg, y = x; di >= 0; --di) 
	*y++ = FF_ZERO;

    /* Multiply
       -------- */
    for (di = 0; di <= dest->Degree; ++di)
    	for (si = 0; si <= src->Degree; ++si)
	    x[si+di] = FfAdd(x[si+di],FfMul(s[si],d[di]));

    /* Overwrite <dest> with the result
       -------------------------------- */
    SysFree(dest->Data);
    dest->Data = x;
    dest->Degree = xdeg;
    dest->BufSize = xdeg+1;
    return dest;
}

