/* ============================= C MeatAxe ==================================
   File:        $Id: settest.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Check if a set contains an element.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.set
!function SetContains "Check if a number is in a set"
!synopsis 
    int SetContains(const Set_t *set, long elem);
!arg set
    Pointer to the set.
!arg elem
    Number to check.
!return
    1 if |set| contains |elem|, 0 else.
!description
    This function checks if |set| contains the number |elem|.
!seealso SetInsert
-------------------------------------------------------------------------- }}}*/

int SetContains(const Set_t *set, long elem)

{
    int i;
    long *l;
 
    if (!SetIsValid(set))
    {
	MTX_ERROR1("%E",MTX_ERR_BADARG);
	return -1;
    }
    l = set->Data;
    for (i = set->Size; i > 0 && *l < elem; --i, ++l);
    return (i > 0 && *l == elem);
}
